@@ -5,7 +5,7 @@ knitr::opts_chunk$set(R.options = list(width = 100))
 library(EHR)
 library(lubridate)
 library(pkdata)
-options(stringsAsFactors = FALSE)
+oldopt <- options(stringsAsFactors = FALSE)
 
 ## ----niceroutput, echo = FALSE--------------------------------------------------------------------
 findbreaks <- function(x, char = '[ /\\]', charlen = 75) {
@@ -34,8 +34,8 @@ findbreaks <- function(x, char = '[ /\\]', charlen = 75) {
   paste(c(strings[1], paste0('     ', strings[-1])), collapse = '\n')
 }
 
-co <- function(expr) {
-  txt <- capture.output(expr)
+co <- function(expr, type = c('output', 'message')) {
+  txt <- capture.output(expr, type = type)
   cat(findbreaks(txt))
   cat("\n")
 }
@@ -87,7 +87,7 @@ names(creat.data)[1] <- names(ivdose.data)[1] <- "mod_id"
 #      drugname='fent',
 #      check.path=checkDir)
 
-## ----sim-build-pk-iv-fake, echo = FALSE-----------------------------------------------------------
+## ----sim-build-pk-iv-fake, echo = FALSE, message = FALSE------------------------------------------
 co({
 simple_pk_dat <- run_Build_PK_IV(
     conc=conc.data,
@@ -104,7 +104,7 @@ simple_pk_dat <- run_Build_PK_IV(
     lab.columns = list(id = 'mod_id', datetime = 'date.time'),
     drugname='fent',
     check.path=checkDir)
-})
+}, 'message')
 
 ## ----sim-build-pk-iv-out--------------------------------------------------------------------------
 head(simple_pk_dat,15)
@@ -301,7 +301,7 @@ saveRDS(alb.cln, file=file.path(dataDir,"Fentanyl_alb_mod_id.rds"))
 ## ----mod-setup------------------------------------------------------------------------------------
 # set crosswalk option 
 xwalk <- readRDS(file.path(dataDir, "Fentanyl_module_id.rds"))
-options(pkxwalk = 'xwalk')
+oldxwalk <- options(pkxwalk = 'xwalk')
 
 # define parameters
 drugname <- 'fent'
@@ -312,6 +312,7 @@ LLOQ <- 0.05
 exclude_val <- function(x, val=1) { !is.na(x) & x == val }
 
 demo.out <- run_Demo(demo.path = file.path(dataDir, "Fentanyl_demo_mod_id.rds"),
+    demo.columns = list(id = 'mod_id_visit'),
     toexclude = expression(exclude_val(in_hospital_mortality) | exclude_val(add_ecmo)),
     demo.mod.list = list(length_of_icu_stay = 
                         expression(daysDiff(surgery_date, date_icu_dc))))
@@ -345,7 +346,7 @@ demo.out$exclude
 #      weightunit = 'kg',
 #      drugname = drugname)
 
-## ----Pro-Med-Str1-fake, echo=FALSE----------------------------------------------------------------
+## ----Pro-Med-Str1-fake, echo=FALSE, message = FALSE-----------------------------------------------
 co({
 ivdose.out <- run_MedStrI(
     mar.path=file.path(dataDir,"Fentanyl_mar_mod_id.rds"),
@@ -369,7 +370,7 @@ ivdose.out <- run_MedStrI(
     ratewgtunit = 'mcg/kg/hr',
     weightunit = 'kg',
     drugname = drugname)
-    })
+}, 'message')
 
 ## ----Pro-Med-Str1-out-----------------------------------------------------------------------------
 head(ivdose.out)
@@ -387,34 +388,45 @@ eRX.out
 
 ## ----Pro-Drug-Level, eval=FALSE-------------------------------------------------------------------
 #  conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
+#      conc.columns = list(
+#        id = 'mod_id', conc = 'conc.level',
+#        idvisit = 'mod_id_visit', samplinkid = 'mod_id_event'
+#      ),
 #      conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
 #      conc.rename=c(fentanyl_calc_conc = 'conc.level', samp= 'event'),
 #      conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
 #      samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
+#      samp.columns = list(
+#        conclinkid = 'mod_id_event', datetime = 'Sample.Collection.Date.and.Time'
+#      ),
 #      samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
 #      check.path=checkDir,
 #      failmiss_fn = 'MissingConcDate-',
 #      multsets_fn = 'multipleSetsConc-',
 #      faildup_fn = 'DuplicateConc-',
-#      drugname=drugname,
-#      LLOQ=LLOQ,
-#      demo.list=demo.out)
+#      drugname = drugname,
+#      LLOQ = LLOQ,
+#      demo.list = demo.out,
+#      demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'))
 
 ## ----Pro-Drug-Level-fake, echo=FALSE--------------------------------------------------------------
 co({
 conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
+    conc.columns = list(id = 'mod_id', conc = 'conc.level', idvisit = 'mod_id_visit', samplinkid = 'mod_id_event'),
     conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
     conc.rename=c(fentanyl_calc_conc = 'conc.level', samp= 'event'),
     conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
     samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
+    samp.columns = list(conclinkid = 'mod_id_event', datetime = 'Sample.Collection.Date.and.Time'),
     samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
     check.path=checkDir,
     failmiss_fn = 'MissingConcDate-',
     multsets_fn = 'multipleSetsConc-',
     faildup_fn = 'DuplicateConc-',
-    drugname=drugname,
-    LLOQ=LLOQ,
-    demo.list=demo.out)
+    drugname = drugname,
+    LLOQ = LLOQ,
+    demo.list = demo.out,
+    demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'))
 })
 
 ## ----Pro-Drug-Level-out---------------------------------------------------------------------------
@@ -424,41 +436,52 @@ head(conc.out)
 ( fail.miss.conc.date <- read.csv(file.path(checkDir,"failMissingConcDate-fent.csv")) )
 
 ## ----fixdate--------------------------------------------------------------------------------------
-fail.miss.conc.date[,"date.time"] <- c("9/30/2016 09:32","10/1/2016 19:20","10/2/2016 02:04")
+fail.miss.conc.date[,"datetime"] <- c("9/30/2016 09:32","10/1/2016 19:20","10/2/2016 02:04")
 fail.miss.conc.date
  
 write.csv(fail.miss.conc.date, file.path(checkDir,"fixMissingConcDate-fent.csv"))
 
 ## ----Pro-Drug-Level-rerun, eval=FALSE-------------------------------------------------------------
 #  conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
+#      conc.columns = list(
+#        id = 'mod_id', conc = 'conc.level',
+#        idvisit = 'mod_id_visit', samplinkid = 'mod_id_event'
+#      ),
 #      conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
 #      conc.rename=c(fentanyl_calc_conc = 'conc.level', samp = 'event'),
 #      conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
 #      samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
+#      samp.columns = list(
+#        conclinkid = 'mod_id_event', datetime = 'Sample.Collection.Date.and.Time'
+#      ),
 #      samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
 #      check.path=checkDir,
 #      failmiss_fn = 'MissingConcDate-',
 #      multsets_fn = 'multipleSetsConc-',
 #      faildup_fn = 'DuplicateConc-',
-#      drugname=drugname,
-#      LLOQ=LLOQ,
-#      demo.list=demo.out)
+#      drugname = drugname,
+#      LLOQ = LLOQ,
+#      demo.list = demo.out,
+#      demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'))
 
 ## ----Pro-Drug-Level-rerun-fake, echo=FALSE--------------------------------------------------------
 co({
 conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
+    conc.columns = list(id = 'mod_id', conc = 'conc.level', idvisit = 'mod_id_visit', samplinkid = 'mod_id_event'),
     conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
     conc.rename=c(fentanyl_calc_conc = 'conc.level', samp = 'event'),
     conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
     samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
+    samp.columns = list(conclinkid = 'mod_id_event', datetime = 'Sample.Collection.Date.and.Time'),
     samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
     check.path=checkDir,
     failmiss_fn = 'MissingConcDate-',
     multsets_fn = 'multipleSetsConc-',
     faildup_fn = 'DuplicateConc-', 
-    drugname=drugname,
-    LLOQ=LLOQ,
-    demo.list=demo.out)
+    drugname = drugname,
+    LLOQ = LLOQ,
+    demo.list = demo.out,
+    demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'))
 })
 
 ## ----remove-fix, include=FALSE--------------------------------------------------------------------
@@ -505,7 +528,7 @@ str(lab.out)
 #      date.format="%m/%d/%y %H:%M:%S",
 #      date.tz="America/Chicago")
 
-## ----Build-PK-IV-fake, echo=FALSE-----------------------------------------------------------------
+## ----Build-PK-IV-fake, echo=FALSE, message=FALSE--------------------------------------------------
 co({
 pk_dat <- run_Build_PK_IV(
     conc=conc.out,
@@ -525,7 +548,7 @@ pk_dat <- run_Build_PK_IV(
     faildupbol_fn='DuplicateBolus-',
     date.format="%m/%d/%y %H:%M:%S",
     date.tz="America/Chicago")
-})
+}, 'message')
 
 ## ----Build-PK-IV-out------------------------------------------------------------------------------
 # convert id back to original IDs
@@ -534,6 +557,9 @@ pk_dat <- pullRealId(pk_dat, remove.mod.id=TRUE)
 head(pk_dat)
 
 ## ----cleanup, echo = FALSE------------------------------------------------------------------------
+# restore old options
+options(oldopt)
+options(oldxwalk)
 # normally, you would not delete these files
 # CRAN policy states that a package should do proper cleanup
 to_delete <- c(file.path(td, 'check1'), file.path(td, 'check2'), file.path(td, 'data2'))
